@@ -4,43 +4,45 @@ from __future__ import division
 import time
 import math
 import logging
+import threading
 import struct
+import threading
 
 from pymycobot.log import setup_logging
 from pymycobot.Interface import MyBuddyCommandGenerator
 from pymycobot.common import ProtocolCode, write, read
 from pymycobot.error import calibration_parameters
+from pymycobot.sms import sms_sts
 
-
-class MyBuddy(MyBuddyCommandGenerator):
+class MyBuddy(MyBuddyCommandGenerator, sms_sts):
     """MyCobot Python API Serial communication class.
 
     Supported methods:
 
         # Overall status
-            Look at parent class: `MyCobotCommandGenerator`.
+            Look at parent class: `CommandGenerator`.
 
         # MDI mode and operation
             get_radians()
             send_radians()
             sync_send_angles() *
             sync_send_coords() *
-            Other look at parent class: `MyCobotCommandGenerator`.
+            Other look at parent class: `CommandGenerator`.
 
         # JOG mode and operation
-            Look at parent class: `MyCobotCommandGenerator`.
+            Look at parent class: `CommandGenerator`.
 
         # Running status and Settings
-            Look at parent class: `MyCobotCommandGenerator`.
+            Look at parent class: `CommandGenerator`.
 
         # Servo control
-            Look at parent class: `MyCobotCommandGenerator`.
+            Look at parent class: `CommandGenerator`.
 
         # Atom IO
-            Look at parent class: `MyCobotCommandGenerator`.
+            Look at parent class: `CommandGenerator`.
 
         # Basic
-            Look at parent class: `MyCobotCommandGenerator`.
+            Look at parent class: `CommandGenerator`.
 
         # Other
             wait() *
@@ -63,6 +65,8 @@ class MyBuddy(MyBuddyCommandGenerator):
         self._serial_port.timeout = timeout
         self._serial_port.rts = False
         self._serial_port.open()
+        self.lock = threading.Lock()
+        super(sms_sts, self).__init__(self._serial_port, 0)
 
     _write = write
     
@@ -106,14 +110,9 @@ class MyBuddy(MyBuddyCommandGenerator):
                             datas = b'\xfe'
                             pre = k  
             except:
-                self.log.debug("_read: {}".format(datas))
-                
-                datas = None
                 break
         else:
-            self.log.debug("_read: {}".format(datas))
-
-            datas = None
+            datas = b''
         self.log.debug("_read: {}".format(datas))
         return datas
     
@@ -131,72 +130,80 @@ class MyBuddy(MyBuddyCommandGenerator):
         """
         real_command, has_reply = super(
             MyBuddy, self)._mesg(genre, *args, **kwargs)
-        self._write(self._flatten(real_command))
+        with self.lock:
+            self._write(self._flatten(real_command))
 
-        if has_reply:
-            data = self._read()
-            res = self._process_received(data, genre, arm=12)
-            if genre in [
-                ProtocolCode.ROBOT_VERSION,
-                ProtocolCode.SOFTWARE_VERSION,
-                ProtocolCode.GET_ROBOT_ID,
-                ProtocolCode.IS_POWER_ON,
-                ProtocolCode.IS_CONTROLLER_CONNECTED,
-                ProtocolCode.IS_PAUSED,  # TODO have bug: return b''
-                ProtocolCode.IS_IN_POSITION,
-                ProtocolCode.IS_MOVING,
-                ProtocolCode.IS_SERVO_ENABLE,
-                ProtocolCode.IS_ALL_SERVO_ENABLE,
-                ProtocolCode.GET_SERVO_DATA,
-                ProtocolCode.GET_DIGITAL_INPUT,
-                ProtocolCode.GET_GRIPPER_VALUE,
-                ProtocolCode.IS_GRIPPER_MOVING,
-                ProtocolCode.GET_SPEED,
-                ProtocolCode.GET_ENCODER,
-                ProtocolCode.GET_BASIC_INPUT,
-                ProtocolCode.GET_TOF_DISTANCE,
-                ProtocolCode.GET_END_TYPE,
-                ProtocolCode.GET_MOVEMENT_TYPE,
-                ProtocolCode.GET_REFERENCE_FRAME,
-                ProtocolCode.GET_JOINT_MIN_ANGLE,
-                ProtocolCode.GET_JOINT_MAX_ANGLE,
-                ProtocolCode.GET_FRESH_MODE,
-                # ProtocolCode.GET_SERVO_CURRENTS
-            ]:
-                return self._process_single(res)
-            elif genre in [ProtocolCode.GET_ANGLES]:
-                return [self._int2angle(angle) for angle in res]
-            elif genre in [ProtocolCode.GET_ANGLE]:
-                return self._int2angle(res[0]) if res else None
-            elif genre in [ProtocolCode.GET_COORD]:
-                if real_command[5] < 4:
-                    if real_command[2] == 3:
-                        return self._int2angle(res[0]) if res else None
-                    return self._int2coord(res[0]) if res else None
-                else:
+            if has_reply:
+                data = self._read()
+                res = self._process_received(data, genre, arm=12)
+                if res is None:
+                    return None
+                if genre in [
+                    ProtocolCode.ROBOT_VERSION,
+                    ProtocolCode.SOFTWARE_VERSION,
+                    ProtocolCode.GET_ROBOT_ID,
+                    ProtocolCode.IS_POWER_ON,
+                    ProtocolCode.IS_CONTROLLER_CONNECTED,
+                    ProtocolCode.IS_PAUSED,  # TODO have bug: return b''
+                    ProtocolCode.IS_IN_POSITION,
+                    ProtocolCode.IS_MOVING,
+                    ProtocolCode.IS_SERVO_ENABLE,
+                    ProtocolCode.IS_ALL_SERVO_ENABLE,
+                    ProtocolCode.GET_SERVO_DATA,
+                    ProtocolCode.GET_DIGITAL_INPUT,
+                    ProtocolCode.GET_GRIPPER_VALUE,
+                    ProtocolCode.IS_GRIPPER_MOVING,
+                    ProtocolCode.GET_SPEED,
+                    ProtocolCode.GET_ENCODER,
+                    ProtocolCode.GET_BASIC_INPUT,
+                    ProtocolCode.GET_TOF_DISTANCE,
+                    ProtocolCode.GET_END_TYPE,
+                    ProtocolCode.GET_MOVEMENT_TYPE,
+                    ProtocolCode.GET_REFERENCE_FRAME,
+                    ProtocolCode.GET_FRESH_MODE,
+                    ProtocolCode.SetHTSGripperTorque,
+                    ProtocolCode.GetHTSGripperTorque,
+                    ProtocolCode.GetGripperProtectCurrent,
+                    ProtocolCode.InitGripper,
+                    ProtocolCode.SET_FOUR_PIECES_ZERO
+                    # ProtocolCode.GET_SERVO_CURRENTS
+                ]:
+                    return self._process_single(res)
+                elif genre in [ProtocolCode.GET_ANGLES]:
+                    return [self._int2angle(angle) for angle in res]
+                elif genre in [ProtocolCode.GET_ANGLE]:
                     return self._int2angle(res[0]) if res else None
-            elif genre in [ProtocolCode.GET_ALL_BASE_COORDS, ProtocolCode.GET_COORDS, ProtocolCode.GET_TOOL_REFERENCE, ProtocolCode.GET_WORLD_REFERENCE, ProtocolCode.GET_BASE_COORDS, ProtocolCode.GET_BASE_COORD, ProtocolCode.BASE_TO_SINGLE_COORDS]:
-                if res:
-                    r = [] 
-                    for idx in range(3):
-                        r.append(self._int2coord(res[idx]))
-                    for idx in range(3, 6):
-                        r.append(self._int2angle(res[idx]))
-                    if len(res) == 12:
-                        r1 = []
-                        for idx in range(6, 9):
-                            r1.append(self._int2coord(res[idx]))
-                        for idx in range(9, 12):
-                            r1.append(self._int2angle(res[idx]))
-                        return [r, r1]
-                    return r
+                elif genre in [ProtocolCode.GET_COORD]:
+                    if real_command[5] < 4:
+                        if real_command[2] == 3:
+                            return self._int2angle(res[0]) if res else None
+                        return self._int2coord(res[0]) if res else None
+                    else:
+                        return self._int2angle(res[0]) if res else None
+                elif genre in [ProtocolCode.GET_ALL_BASE_COORDS, ProtocolCode.GET_COORDS, ProtocolCode.GET_TOOL_REFERENCE, ProtocolCode.GET_WORLD_REFERENCE, ProtocolCode.GET_BASE_COORDS, ProtocolCode.GET_BASE_COORD, ProtocolCode.BASE_TO_SINGLE_COORDS]:
+                    if res:
+                        r = [] 
+                        for idx in range(3):
+                            r.append(self._int2coord(res[idx]))
+                        for idx in range(3, 6):
+                            r.append(self._int2angle(res[idx]))
+                        if len(res) == 12:
+                            r1 = []
+                            for idx in range(6, 9):
+                                r1.append(self._int2coord(res[idx]))
+                            for idx in range(9, 12):
+                                r1.append(self._int2angle(res[idx]))
+                            return [r, r1]
+                        return r
+                    else:
+                        return res
+                elif genre in [ProtocolCode.GET_JOINT_MAX_ANGLE, ProtocolCode.GET_JOINT_MIN_ANGLE]:
+                    return self._int2coord(res[0])
+                elif genre in [ProtocolCode.GET_SERVO_VOLTAGES, ProtocolCode.COLLISION]:
+                    return [self._int2coord(angle) for angle in res]
                 else:
                     return res
-            elif genre in [ProtocolCode.GET_SERVO_VOLTAGES, ProtocolCode.COLLISION]:
-                return [self._int2coord(angle) for angle in res]
-            else:
-                return res
-        return None
+            return None
 
     def get_radians(self, id):
         """Get the radians of all joints
@@ -263,9 +270,9 @@ class MyBuddy(MyBuddyCommandGenerator):
         """
         pin_no = self.base_io_to_gpio(pin_no)       
         if mode == 1:
-            self.gpio.setup(pin_no, self.gpio.OUT, initial=self.gpio.HIGH)
+            self.gpio.setup(pin_no, self.gpio.OUT)
         else:
-            self.gpio.setup(pin_no, self.gpio.IN, pull_up_down=self.gpio.DOWN)
+            self.gpio.setup(pin_no, self.gpio.IN)
             
     def set_gpio_output(self, pin_no, v):
         """Set GPIO output value.
@@ -284,7 +291,7 @@ class MyBuddy(MyBuddyCommandGenerator):
             pin_no: (int)pin number 1-16.
         """
         pin = self.base_io_to_gpio(pin_no)
-        self.gpio.input(pin)
+        return self.gpio.input(pin)
         
     def set_gpio_pwm_start(self, pin_no, freq = 0.5, dc = 0.5):
         """Set GPIO PWM value.
@@ -395,4 +402,40 @@ class MyBuddy(MyBuddyCommandGenerator):
         self._serial_port.open()
         
     def is_open(self):
-        return self._serial_port.isOpen()
+        return self._serial_port.is_open
+    
+    def sync_send_angles(self, id, degrees, speed, timeout=15):
+        """Send the angle in synchronous state and return when the target point is reached
+            
+        Args:
+            degrees: a list of degree values(List[float]), length 6.
+            speed: (int) 0 ~ 100
+            timeout: default 7s.
+        """
+        t = time.time()
+        self.send_angles(id, degrees, speed)
+        time.sleep(0.5)
+        while time.time() - t < timeout:
+            f = self.is_in_position(id, degrees, 0)
+            if f == 1:
+                break
+            time.sleep(0.1)
+        return self
+
+    def sync_send_coords(self, id, coords, speed, mode=0, timeout=15):
+        """Send the coord in synchronous state and return when the target point is reached
+            
+        Args:
+            coords: a list of coord values(List[float])
+            speed: (int) 0 ~ 100
+            mode: (int): 0 - angular（default）, 1 - linear
+            timeout: default 7s.
+        """
+        t = time.time()
+        self.send_coords(id, coords, speed, mode)
+        time.sleep(0.5)
+        while time.time() - t < timeout:
+            if self.is_in_position(id, coords, 1) == 1:
+                break
+            time.sleep(0.1)
+        return self
