@@ -1,13 +1,14 @@
 # coding=utf-8
 
-import logging
+import threading
 import math
 import time
+import threading
 
 from .log import setup_logging
-from .generate import MyCobotCommandGenerator
+from .generate import CommandGenerator
 from .common import ProtocolCode, read, write
-
+from pymycobot.sms import sms_sts
 
 class MyPalletizedataException(Exception):
     pass
@@ -80,7 +81,7 @@ def calibration_parameters(**kwargs):
                 )
 
 
-class MyPalletizer(MyCobotCommandGenerator):
+class MyPalletizer(CommandGenerator, sms_sts):
     def __init__(self, port, baudrate="115200", timeout=0.1, debug=False):
         """
         Args:
@@ -100,6 +101,8 @@ class MyPalletizer(MyCobotCommandGenerator):
         self._serial_port.timeout = timeout
         self._serial_port.rts = False
         self._serial_port.open()
+        self.lock = threading.Lock()
+        super(sms_sts, self).__init__(self._serial_port, 0)
 
     _write = write
     _read = read
@@ -119,53 +122,75 @@ class MyPalletizer(MyCobotCommandGenerator):
         real_command, has_reply = super(MyPalletizer, self)._mesg(
             genre, *args, **kwargs
         )
-        self._write(self._flatten(real_command))
+        with self.lock:
+            self._write(self._flatten(real_command))
 
-        if has_reply:
-            data = self._read(genre)
-            # print(data)
-            res = self._process_received(data, genre)
-            if genre in [
-                ProtocolCode.ROBOT_VERSION,
-                ProtocolCode.SOFTWARE_VERSION,
-                ProtocolCode.GET_ROBOT_ID,
-                ProtocolCode.IS_POWER_ON,
-                ProtocolCode.IS_CONTROLLER_CONNECTED,
-                ProtocolCode.IS_PAUSED,
-                ProtocolCode.IS_IN_POSITION,
-                ProtocolCode.IS_MOVING,
-                ProtocolCode.IS_SERVO_ENABLE,
-                ProtocolCode.IS_ALL_SERVO_ENABLE,
-                ProtocolCode.GET_SERVO_DATA,
-                ProtocolCode.GET_DIGITAL_INPUT,
-                ProtocolCode.GET_GRIPPER_VALUE,
-                ProtocolCode.IS_GRIPPER_MOVING,
-                ProtocolCode.GET_SPEED,
-                ProtocolCode.GET_ENCODER,
-                ProtocolCode.GET_BASIC_INPUT,
-                ProtocolCode.GET_TOF_DISTANCE
-            ]:
-                return self._process_single(res)
-            elif genre in [ProtocolCode.GET_ANGLES]:
-                return [self._int2angle(angle) for angle in res]
-            elif genre in [ProtocolCode.GET_COORDS]:
-                if res:
+            if has_reply:
+                data = self._read(genre)
+                # print(data)
+                res = self._process_received(data, genre)
+                if res == []:
+                    return None
+                if genre in [
+                    ProtocolCode.ROBOT_VERSION,
+                    ProtocolCode.SOFTWARE_VERSION,
+                    ProtocolCode.GET_ROBOT_ID,
+                    ProtocolCode.IS_POWER_ON,
+                    ProtocolCode.IS_CONTROLLER_CONNECTED,
+                    ProtocolCode.IS_PAUSED,
+                    ProtocolCode.IS_IN_POSITION,
+                    ProtocolCode.IS_MOVING,
+                    ProtocolCode.IS_SERVO_ENABLE,
+                    ProtocolCode.IS_ALL_SERVO_ENABLE,
+                    ProtocolCode.GET_SERVO_DATA,
+                    ProtocolCode.GET_DIGITAL_INPUT,
+                    ProtocolCode.GET_GRIPPER_VALUE,
+                    ProtocolCode.IS_GRIPPER_MOVING,
+                    ProtocolCode.GET_SPEED,
+                    ProtocolCode.GET_ENCODER,
+                    ProtocolCode.GET_BASIC_INPUT,
+                    ProtocolCode.GET_TOF_DISTANCE,
+                    ProtocolCode.GET_COMMUNICATE_MODE,
+                    ProtocolCode.SET_COMMUNICATE_MODE,
+                    ProtocolCode.SetHTSGripperTorque,
+                    ProtocolCode.GetHTSGripperTorque,
+                    ProtocolCode.GetGripperProtectCurrent,
+                    ProtocolCode.InitGripper,
+                    ProtocolCode.SET_FOUR_PIECES_ZERO
+                ]:
+                    return self._process_single(res)
+                elif genre in [ProtocolCode.GET_ANGLES]:
+                    return [self._int2angle(angle) for angle in res]
+                elif genre in [ProtocolCode.GET_COORDS]:
+                    if res:
+                        r = []
+                        for idx in range(3):
+                            r.append(self._int2coord(res[idx]))
+                        if len(res)>3:
+                            r.append(self._int2angle(res[3]))
+                        return r
+                    else:
+                        return res
+                elif genre in [
+                    ProtocolCode.GET_JOINT_MIN_ANGLE,
+                    ProtocolCode.GET_JOINT_MAX_ANGLE,
+                ]:
+                    return self._int2angle(res[0]) if res else 0
+                elif genre in [ProtocolCode.GET_BASIC_VERSION, ProtocolCode.SOFTWARE_VERSION, ProtocolCode.GET_ATOM_VERSION]:
+                    return self._int2coord(self._process_single(res))
+                elif genre == ProtocolCode.GET_ANGLES_COORDS:
                     r = []
-                    for idx in range(3):
-                        r.append(self._int2coord(res[idx]))
-                    if len(res)>3:
-                        r.append(self._int2angle(res[3]))
+                    for index in range(len(res)):
+                        if index < 4:
+                            r.append(self._int2angle(res[index]))
+                        elif index < 7:
+                            r.append(self._int2coord(res[index]))
+                        else:
+                            r.append(self._int2angle(res[index]))
                     return r
                 else:
                     return res
-            elif genre in [
-                ProtocolCode.GET_JOINT_MIN_ANGLE,
-                ProtocolCode.GET_JOINT_MAX_ANGLE,
-            ]:
-                return self._int2angle(res[0]) if res else 0
-            else:
-                return res
-        return None
+            return None
 
     def get_radians(self):
         """Get all angle return a list
@@ -188,7 +213,7 @@ class MyPalletizer(MyCobotCommandGenerator):
                    for radian in radians]
         return self._mesg(ProtocolCode.SEND_ANGLES, degrees, speed)
 
-    def sync_send_angles(self, degrees, speed, timeout=7):
+    def sync_send_angles(self, degrees, speed, timeout=15):
         t = time.time()
         self.send_angles(degrees, speed)
         while time.time() - t < timeout:
@@ -198,7 +223,7 @@ class MyPalletizer(MyCobotCommandGenerator):
             time.sleep(0.1)
         return self
 
-    def sync_send_coords(self, coords, speed, mode, timeout=7):
+    def sync_send_coords(self, coords, speed, mode, timeout=15):
         t = time.time()
         self.send_coords(coords, speed, mode)
         while time.time() - t < timeout:
@@ -242,3 +267,9 @@ class MyPalletizer(MyCobotCommandGenerator):
         """
         data_list = [[25, 21], [26, 32]]
         return self._mesg(ProtocolCode.GET_ACCEI_DATA, data_list[1] if value else data_list[0], has_reply=True)
+    
+    def close(self):
+        self._serial_port.close()
+        
+    def open(self):
+        self._serial_port.open()
