@@ -228,8 +228,8 @@ class RobotLimit:
     robot_limit = {
         "Mercury":{
             "joint_id":[1,2,3,4,5,6,7,11,12,13],
-            "angles_min":[-165, -50, -165, -165, -165, -20, -180, -60, -138, -115],
-            "angles_max":[165, 120, 165, 5, 165, 265, 180, 0, 175, 115],
+            "angles_min":[-165, -50, -165, -165, -165, -75, -165, -55, -70, -160],
+            "angles_max":[165, 120, 165, 1, 165, 255, 165, 0, 245, 160],
             "coords_min":[-459, -459, -300, -180, -180, -180],
             "coords_max":[459, 459, 542, 180, 180, 180],
             "left_coords_min":[-351.11, -272.12, -262.91, -180, -180, -180],
@@ -239,8 +239,8 @@ class RobotLimit:
         },
         "MercurySocket":{
             "joint_id":[1,2,3,4,5,6,7,11,12,13],
-            "angles_min":[-165, -50, -165, -165, -165, -20, -180, -60, -138, -115],
-            "angles_max":[165, 120, 165, 5, 165, 265, 180, 0, 175, 115],
+            "angles_min":[-165, -50, -165, -165, -165, -75, -165, -55, -70, -160],
+            "angles_max":[165, 120, 165, 1, 165, 255, 165, 0, 245, 160],
             "coords_min":[-459, -459, -300, -180, -180, -180],
             "coords_max":[459, 459, 542, 180, 180, 180],
             "left_coords_min":[-351.11, -272.12, -262.91, -180, -180, -180],
@@ -399,5 +399,14 @@ class RobotLimit:
             "left_coords_max": [250, 260, 480, 180, 180, 180],
             "right_coords_min": [0, -260, 0, -180, -180, -180],
             "right_coords_max": [250, 40, 480, 180, 180, 180]
+        },
+        "Pro630": {
+            "joint_id": [1, 2, 3, 4, 5, 6],
+            "servo_id": [1, 2, 3, 4, 5, 6],
+            "coord_id": (1, 2, 3, 4, 5, 6),
+            "angles_min": [-180, -45, -165, -90, -180, -180],
+            "angles_max": [180, 225, 165, 270, 180, 180],
+            "coords_min": [-630, -630, -425, -180, -180, -180],
+            "coords_max": [630, 630, 835, 180, 180, 180]
         }
     }
